/*
 * Copyright 2020 ETSI
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package fr.emse.gitlab.saref.checkers;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import org.apache.jena.query.Dataset;
import org.apache.jena.query.QueryExecution;
import org.apache.jena.query.QueryExecutionFactory;
import org.apache.jena.query.QuerySolution;
import org.apache.jena.query.ReadWrite;
import org.apache.jena.query.ResultSet;
import org.apache.jena.rdf.model.Literal;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.rdf.model.RDFNode;
import org.apache.jena.rdf.model.Resource;
import org.apache.jena.rdf.model.ResourceFactory;
import org.topbraid.jenax.util.JenaUtil;
import org.topbraid.shacl.validation.ValidationUtil;
import org.topbraid.shacl.vocabulary.SH;

import fr.emse.gitlab.saref.SAREF;
import fr.emse.gitlab.saref.SAREFPipeline.Mode;
import fr.emse.gitlab.saref.SAREFPipelineException;
import fr.emse.gitlab.saref.managers.RepositoryManager;
import fr.emse.gitlab.saref.vocabs.SHACL;

public abstract class AbstractShaclChecker extends AbstractClauseChecker {

	public static final String SHAPE_VAR = "shape";
	public static final Pattern SHAPE_PATTERN = Pattern.compile("^(?<shape>Clause((_[0-9]+)+))_Checker$");
	private static final String NS = "https://saref.etsi.org/shape#";

	protected enum Severity {
		WARNING(SHACL.Warning), VIOLATION(SHACL.Violation);

		private final Resource resource;

		private Severity(Resource resource) {
			this.resource = resource;
		}

		public Resource getResource() {
			return resource;
		}
	}

	protected static interface MessageResource {
		default Resource asResource() {
			return ResourceFactory.createResource(NS + toString());
		}
	}

	protected static final String SELECT_VIOLATION = "PREFIX sh: <http://www.w3.org/ns/shacl#>\n"
			+ "SELECT ?focusNode ?resultMessage ?resultPath ?value ?severity\n" + "WHERE {   \n"
			+ "    ?violation sh:focusNode ?focusNode .\n" + "    ?violation sh:resultMessage ?resultMessage .\n"
			+ "    ?violation sh:resultSeverity ?severity .\n"
			+ "      OPTIONAL { ?violation sh:sourceShape ?sourceShape . }\n"
			+ "      OPTIONAL { ?violation sh:resultPath ?resultPath . }\n"
			+ "      OPTIONAL { ?violation sh:value ?value . } \n" + "}"
			+ "ORDER BY ?severity ?sourceShape ?resultMessage ?focusNode ";

	protected final Model shapeModel;
	protected final Severity defaultSeverity;

	public AbstractShaclChecker(RepositoryManager repositoryManager, Class<?> clazz, String name)
			throws SAREFPipelineException {
		super(repositoryManager, clazz, name);
		shapeModel = JenaUtil.createDefaultModel();

		final Matcher matcher = SHAPE_PATTERN.matcher(clazz.getSimpleName());
		if (!matcher.find()) {
			throw new IllegalArgumentException();
		}
		final String shape = String.format("shapes/%s.ttl", matcher.group(SHAPE_VAR));
		try (InputStream in = AbstractShaclChecker.class.getClassLoader().getResourceAsStream(shape)) {
			shapeModel.read(in, SAREF.BASE, "TTL");
		} catch (Exception ex) {
			throw new SAREFPipelineException("Exception while reading the shape file", ex);
		}
		if (pipeline.mode == Mode.DEVELOP) {
			defaultSeverity = Severity.WARNING;
		} else {
			defaultSeverity = Severity.VIOLATION;
		}
	}

	public AbstractShaclChecker(RepositoryManager repositoryManager, Class<?> clazz) throws SAREFPipelineException {
		this(repositoryManager, clazz, null);
	}

	protected abstract void updateShapeModel();

	@Override
	public final void check() throws SAREFPipelineException {
		Dataset dataset = datasetManager.getDataset();
		dataset.begin(ReadWrite.READ);
		Model model = datasetManager.getModel(version);

		updateShapeModel();

		Resource reportResource = ValidationUtil.validateModel(model, shapeModel, true);
		boolean conforms = reportResource.getProperty(SH.conforms).getBoolean();
		if (!conforms) {
			Model reportModel = reportResource.getModel();
//			reportModel.setNsPrefixes(PREFIXES);
//			reportModel.setNsPrefix("sh", "http://www.w3.org/ns/shacl#");
			try (QueryExecution exec = QueryExecutionFactory.create(SELECT_VIOLATION, reportModel);) {
				Resource previousSeverity = null;
				Literal previousResultMessage = null;
				List<Resource> previousFocusNodes = new ArrayList<>();
				List<RDFNode> previousvalues = new ArrayList<>();
				for (ResultSet resultSet = exec.execSelect(); resultSet.hasNext();) {
					QuerySolution sol = resultSet.next();
					Resource severity = sol.getResource("severity");
					Literal resultMessage = sol.getLiteral("resultMessage");
					Resource focusNode = sol.get("focusNode").asResource();
					RDFNode value = sol.get("value");

					if (severity != null && severity.equals(previousSeverity) && resultMessage != null
							&& resultMessage.equals(previousResultMessage)) {
						previousFocusNodes.add(focusNode);
						previousvalues.add(value);
					} else {
						if (previousSeverity != null) {
							String message = String.format("%s: %s", previousResultMessage,
									String.join(" - ", previousFocusNodes.stream().map(Object::toString)
											.collect(Collectors.toList()).toArray(new String[1])));
							if (severity.equals(SHACL.Violation)) {
								errorLogger.error(message);
							} else {
								errorLogger.warn(message);
							}
						}
						previousSeverity = severity;
						previousResultMessage = resultMessage;
						previousFocusNodes = new ArrayList<>();
						previousvalues = new ArrayList<>();
					}
				}
				if (!previousFocusNodes.isEmpty()) {
					String message = String.format("%s: %s", previousResultMessage,
							String.join(" - ", previousFocusNodes.stream().map(Object::toString)
									.collect(Collectors.toList()).toArray(new String[1])));
					if (previousSeverity != null && previousSeverity.equals(SHACL.Violation)) {
						errorLogger.error(message);
					} else {
						errorLogger.warn(message);
					}
				}
			}
		}

		dataset.end();
	}

	protected void remove(MessageResource message) {
		shapeModel.remove(message.asResource(), null, null);
		shapeModel.remove(null, null, message.asResource());
	}

	protected void add(MessageResource message, Object... args) {
		add(null, null, message, args);
	}
	
	protected void add(Severity severity, MessageResource message, Object... args) {
		add(severity, null, message, args);
	}
	
	protected void add(String pattern, MessageResource message, Object... args) {
		add(null, pattern, message, args);
	}

	protected void add(Severity severity, String pattern, MessageResource message, Object... args) {
		Resource r = message.asResource();
		if (severity != null) {
			shapeModel.add(r, SHACL.severity, severity.getResource());
		}
		if (pattern != null) {
			shapeModel.add(r, SHACL.pattern, pattern);
		}
		shapeModel.add(r, SHACL.message, getMessage(message, args));
	}

}