/*
 * Copyright 2020 ETSI
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package fr.emse.gitlab.saref.checkers;

import java.io.File;
import java.io.IOException;
import java.nio.file.FileSystems;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.PathMatcher;
import java.util.Optional;

import fr.emse.gitlab.saref.SAREFPipeline.Mode;
import fr.emse.gitlab.saref.SAREFPipelineException;
import fr.emse.gitlab.saref.managers.RepositoryManager;

/**
 * Checks TS 103 673 Clause 9.3: Ontology requirements
 * 
 */
public class Clause_9_3_Checker extends AbstractClauseChecker{

	private static final PathMatcher csvMatcher = FileSystems.getDefault().getPathMatcher("glob:*.csv");
	private static final String FIRST_LINE = "Id;Category;Requirement";

	private static enum MESSAGE {
		missing, ioexception, line;
	}

	public Clause_9_3_Checker(RepositoryManager repositoryManager) {
		super(repositoryManager, Clause_9_3_Checker.class);
	}

	@Override
	public void check() throws SAREFPipelineException {
		File dir = new File(repository.getDirectory(), "requirements");
		if (!dir.isDirectory()) {
			return;
		}
		try {
			checkExists(dir.toPath());
			checkFirstLine(dir.toPath());
		} catch (IOException ex) {
			log(getMessage(MESSAGE.ioexception), Mode.DEVELOP, Mode.RELEASE, Mode.PORTAL);
		}
	}

	private void checkExists(Path path) throws IOException {
		boolean containsFile = Files.walk(path).anyMatch(p -> {
			return csvMatcher.matches(p);
		});
		if (!containsFile) {
			log(getMessage(MESSAGE.missing), Mode.DEVELOP, Mode.RELEASE, Mode.PORTAL);
		}
	}

	private void checkFirstLine(Path path) throws IOException {
		Files.walk(path).filter(p -> {
			return csvMatcher.matches(p);
		}).forEach(p->{
			Optional<String> firstLine;
			try {
				firstLine = Files.lines(p).findFirst();
				if(!firstLine.isPresent()) {
					log(getMessage(MESSAGE.line, p.getFileName()), Mode.DEVELOP, Mode.RELEASE, Mode.PORTAL);
				}
				if(!firstLine.get().equals(FIRST_LINE)) {
					log(getMessage(MESSAGE.line, p.getFileName()), Mode.DEVELOP, Mode.RELEASE, Mode.PORTAL);
				}
			} catch (IOException e) {
				log(getMessage(MESSAGE.ioexception), Mode.DEVELOP, Mode.RELEASE, Mode.PORTAL);
			}
		});
	}

}
