/*
 * Copyright 2020 ETSI
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package fr.emse.gitlab.saref.checkers;

import java.util.HashSet;
import java.util.Set;
import java.util.regex.Pattern;

import org.apache.jena.query.Dataset;
import org.apache.jena.query.ReadWrite;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.rdf.model.Resource;
import org.apache.jena.vocabulary.OWL2;
import org.apache.jena.vocabulary.RDF;
import org.apache.jena.vocabulary.RDFS;

import fr.emse.gitlab.saref.SAREF;
import fr.emse.gitlab.saref.SAREFPipeline.Mode;
import fr.emse.gitlab.saref.SAREFPipelineException;
import fr.emse.gitlab.saref.entities.SAREFVersion;
import fr.emse.gitlab.saref.managers.RepositoryManager;
import fr.emse.gitlab.saref.vocabs.EX;
import fr.emse.gitlab.saref.vocabs.RDFP;

/**
 * Checks TS 103 673 Clause 9.4.4.1: Term IRI
 * 
 */
public class Clause_9_4_4_1_Checker extends AbstractClauseChecker {

	private static final Pattern PATTERN_LOCAL_NAME = Pattern.compile("^[a-zA-Z][a-zA-Z0-9]+$");
	private static final Pattern PATTERN_CAMEL_CASE = Pattern.compile("^[A-Z][a-zA-Z0-9]+$");
	private static final Pattern PATTERN_MIXED_CASE = Pattern.compile("^[a-z][a-zA-Z0-9]+$");

	private static enum MESSAGE {
		pattern, classs, namedindividual, objectproperty, datatypeproperty
	}

	public Clause_9_4_4_1_Checker(RepositoryManager repositoryManager) {
		super(repositoryManager, Clause_9_4_4_1_Checker.class);
	}

	@Override
	public void check() throws SAREFPipelineException {
		Dataset dataset = datasetManager.getDataset();
		dataset.begin(ReadWrite.WRITE);
		Model model = datasetManager.getModel(version);
		Model config = datasetManager.getConfig();

		Set<Resource> definedTerms = new HashSet<>();
		Set<Resource> usedTerms = new HashSet<>();
		computeTerms(version, model, definedTerms, usedTerms);

		Set<Resource> termsBadPattern = new HashSet<>();
		Set<Resource> classesBadCase = new HashSet<>();
		Set<Resource> namedIndividualsBadCase = new HashSet<>();
		Set<Resource> objectPropertiesBadCase = new HashSet<>();
		Set<Resource> datatypePropertiesBadCase = new HashSet<>();

		for (Resource t : definedTerms) {
			String localName = t.getURI().substring(version.getRepository().getProject().getNamespace().length());

			config.add(t, RDF.type, RDFP.Resource);
			config.add(t, EX.localName, localName);
			config.add(t, RDFS.isDefinedBy, version.getVersionResource());

//			Model termModel = datasetManager.getModel(t.getURI());
//			termModel.add(t, RDF.type, RDFS.Resource);
//			termModel.add(t, RDFS.isDefinedBy, version.getResource());
//			// keep the most recent definition of the term
//			if (version.getNextVersion() == null) {
//				termModel.add(DCTerms.modified, RDF.type, OWL.AnnotationProperty);
//				termModel.add(t, DCTerms.modified, SIMPLE_DATE_FORMAT.format(version.getIssued()), XSDDatatype.XSDdate);
//				try (QueryExecution exec = QueryExecutionFactory.create("DESCRIBE <" + t.getURI() + ">", model)) {
//					termModel.add(exec.execDescribe());
//				}
//			}

			if (!PATTERN_LOCAL_NAME.matcher(localName).matches()) {
				termsBadPattern.add(t);
			}
			if (model.contains(t, RDF.type, OWL2.Class) && !PATTERN_CAMEL_CASE.matcher(localName).matches()) {
				classesBadCase.add(t);
			}
			if (model.contains(t, RDF.type, OWL2.NamedIndividual) && !PATTERN_CAMEL_CASE.matcher(localName).matches()) {
				namedIndividualsBadCase.add(t);
			}
			if (model.contains(t, RDF.type, OWL2.ObjectProperty) && !PATTERN_MIXED_CASE.matcher(localName).matches()) {
				objectPropertiesBadCase.add(t);
			}
			if (model.contains(t, RDF.type, OWL2.DatatypeProperty)
					&& !PATTERN_MIXED_CASE.matcher(localName).matches()) {
				datatypePropertiesBadCase.add(t);
			}
		}

		dataset.commit();

		if (!termsBadPattern.isEmpty()) {
			log(getMessage(MESSAGE.pattern,
					String.join(", ", termsBadPattern.toArray(new String[termsBadPattern.size()]))), Mode.DEVELOP,
					Mode.PORTAL, Mode.RELEASE);
		}
		if (!classesBadCase.isEmpty()) {
			log(getMessage(MESSAGE.classs,
					String.join(", ", classesBadCase.toArray(new String[classesBadCase.size()]))), Mode.DEVELOP,
					Mode.PORTAL, Mode.RELEASE);
		}
		if (!namedIndividualsBadCase.isEmpty()) {
			log(getMessage(MESSAGE.namedindividual,
					String.join(", ", namedIndividualsBadCase.toArray(new String[namedIndividualsBadCase.size()]))),
					Mode.DEVELOP, Mode.PORTAL, Mode.RELEASE);
		}
		if (!objectPropertiesBadCase.isEmpty()) {
			log(getMessage(MESSAGE.objectproperty,
					String.join(", ", objectPropertiesBadCase.toArray(new String[objectPropertiesBadCase.size()]))),
					Mode.DEVELOP, Mode.PORTAL, Mode.RELEASE);
		}
		if (!datatypePropertiesBadCase.isEmpty()) {
			log(getMessage(MESSAGE.datatypeproperty,
					String.join(", ", datatypePropertiesBadCase.toArray(new String[datatypePropertiesBadCase.size()]))),
					Mode.DEVELOP, Mode.PORTAL, Mode.RELEASE);
		}

//		for (Resource t : usedTerms) {
//			Model termModel = datasetManager.getModel(t.getURI());
//			termModel.add(t, EX.isUsedBy, version.getResource());
//			config.add(t, RDF.type, RDFP.Resource);
//			config.add(t, EX.isUsedBy, version.getResource());
//		}
	}

	private void computeTerms(SAREFVersion version, Model model, Set<Resource> definedTerms, Set<Resource> usedTerms) {
		model.listStatements().forEachRemaining(stmt -> {
			Resource s = stmt.getSubject();
			Resource p = stmt.getPredicate();
			Resource o = stmt.getObject().isResource() ? (Resource) stmt.getObject() : null;
			computeTerms(version, s, definedTerms, usedTerms);
			computeTerms(version, p, definedTerms, usedTerms);
			computeTerms(version, o, definedTerms, usedTerms);
		});
	}

	private void computeTerms(SAREFVersion version, Resource t, Set<Resource> definedTerms, Set<Resource> usedTerms) {
		if (t == null || !t.isURIResource() || !t.getURI().startsWith(SAREF.BASE) || t.getURI().endsWith("/")) {
			return;
		}
		String uri = t.getURI();
		if (uri.startsWith(version.getRepository().getProject().getNamespace())) {
			definedTerms.add(t);
		} else {
			usedTerms.add(t);
		}
	}

}