/*
 * Copyright 2020 ETSI
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package fr.emse.gitlab.saref.checkers;

import org.apache.jena.query.Dataset;
import org.apache.jena.query.ReadWrite;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.rdf.model.ResIterator;
import org.apache.jena.vocabulary.DCTypes;
import org.apache.jena.vocabulary.RDF;

import fr.emse.gitlab.saref.SAREF;
import fr.emse.gitlab.saref.SAREFPipeline.Mode;
import fr.emse.gitlab.saref.SAREFPipelineException;
import fr.emse.gitlab.saref.entities.SAREFExample;
import fr.emse.gitlab.saref.managers.RepositoryManager;

/**
 * Checks TS 103 673 Clause 9.6.3: The example declaration
 * 
 */
public class Clause_9_6_3_Checker extends AbstractShaclChecker {

	private static enum MESSAGE implements MessageResource {
		dataset, conformsTo1, conformsTo2, conformsTo3,
		title1, title2, title3, abstract1, description1, description2,
		license
	}

	private final SAREFExample example;

	public Clause_9_6_3_Checker(RepositoryManager repositoryManager, SAREFExample example)
			throws SAREFPipelineException {
		super(repositoryManager, Clause_9_6_3_Checker.class, "example " + example.getName());
		this.example = example;
	}

	protected final void updateShapeModel() {
		Dataset dataset = datasetManager.getDataset();
		dataset.begin(ReadWrite.READ);
		Model model = datasetManager.getModel(example);
		int onto = 0;
		for (ResIterator it = model.listSubjectsWithProperty(RDF.type, DCTypes.Dataset); it.hasNext();) {
			it.next();
			onto++;
		}
		if (onto != 1) {
			String msg = getMessage(MESSAGE.dataset, repository.getNamespace());
			log(msg, Mode.PORTAL, Mode.RELEASE);
			return;
		}

		add(defaultSeverity, version.getVersionIRI(), MESSAGE.conformsTo1, version.getVersionIRI());
		add(defaultSeverity, MESSAGE.conformsTo2);
		add(MESSAGE.conformsTo3);
		add(MESSAGE.title1);
		add(defaultSeverity, MESSAGE.title2);
		add(MESSAGE.title3);
		add(defaultSeverity, MESSAGE.abstract1);
		add(MESSAGE.description1);
		add(defaultSeverity, MESSAGE.description2);
		add(String.format("^%s$", SAREF.LICENSE), MESSAGE.license);
	}

}