/*
 * Copyright 2020 ETSI
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package fr.emse.gitlab.saref.entities;

import java.nio.file.Path;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.jena.rdf.model.Resource;
import org.apache.jena.rdf.model.ResourceFactory;

import fr.emse.gitlab.saref.SAREF;

public class SAREFExample {

	public static final String EXAMPLE_NAME_VAR = "example_name";
	public static final String REGEX_EXAMPLE_NAME = String.format("^(?<%s>(.*)\\.ttl$", EXAMPLE_NAME_VAR);
	public static final Pattern PATTERN_EXAMPLE_NAME = Pattern.compile(REGEX_EXAMPLE_NAME);

	private final SAREFVersion version;
	private final String name;
	private final String fileName;
	private final String namespace;
	private final String iri;
	private final Resource resource;

	public SAREFExample(SAREFVersion version, String fileName) {
		this.version = version;
		this.fileName = fileName;
		Matcher matcher = PATTERN_EXAMPLE_NAME.matcher(fileName);
		if (!matcher.find()) {
			throw new IllegalArgumentException();
		}
		this.name = matcher.group(EXAMPLE_NAME_VAR);
		this.namespace = String.format("%s/example/%s/", SAREF.BASE, version.getVersionIRI(), name);
		this.iri = String.format("%s/example/%s#", SAREF.BASE, version.getVersionIRI(), name);
		this.resource = ResourceFactory.createResource(iri);
	}

	public SAREFExample(SAREFVersion version, Path p) {
		this(version, p.toFile().getName());
	}

	public String getName() {
		return name;
	}

	public SAREFVersion getVersion() {
		return version;
	}

	public SAREFRepository getRepository() {
		return version.getRepository();
	}

	public String getFileName() {
		return fileName;
	}

	public String getIRI() {
		return iri;
	}

	public String getNamespace() {
		return namespace;
	}

	public Resource getResource() {
		return resource;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		result = prime * result + ((version == null) ? 0 : version.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		SAREFExample other = (SAREFExample) obj;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		if (version == null) {
			if (other.version != null)
				return false;
		} else if (!version.equals(other.version))
			return false;
		return true;
	}

}
