/*
 * Copyright 2020 ETSI
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package fr.emse.gitlab.saref.entities;

import org.apache.jena.rdf.model.Resource;
import org.apache.jena.rdf.model.ResourceFactory;
import org.eclipse.jgit.lib.Ref;
	
public class SAREFVersion implements SAREFProject {
	
	private final SAREFRepository repository;
	private final SAREFVersionName versionName;
	private final Ref developRef;
	private final Ref prereleaseRef;
	private final Ref releaseRef;

	public SAREFVersion(SAREFRepository repository, SAREFVersionName versionName, Ref developRef, Ref prereleaseRef, Ref releaseRef) {
		this.repository = repository;
		this.versionName = versionName;
		this.developRef = developRef;
		this.prereleaseRef = prereleaseRef;
		this.releaseRef = releaseRef;
	}
	
	public SAREFRepository getRepository() {
		return repository;
	}
	
	public SAREFVersionName getVersionName() {
		return versionName;
	}
	
	public Ref getDevelopRef() {
		return developRef;
	}
	
	public Ref getPrereleaseRef() {
		return prereleaseRef;
	}
	
	public Ref getReleaseRef() {
		return releaseRef;
	}


	/**
	 * Gets the IRI of this version
	 * @return
	 */
	public String getVersionIRI() {
		if(versionName == null) {
			return null;
		}
		return String.format("%s%s/", repository.getNamespace(), versionName);
	}

	/**
	 * Gets the Jena Resource for this version
	 * @return
	 */
	public Resource getVersionResource() {
		if(versionName == null) {
			return null;
		}
		return ResourceFactory.createResource(getVersionIRI());
	}

	@Override
	public String getName() {
		return repository.getName();
	}

	@Override
	public String getRepositoryName() {
		return repository.getRepositoryName();
	}

	@Override
	public String getPrefix() {
		return repository.getPrefix();
	}

	@Override
	public String getPath() {
		return repository.getPath();
	}

	@Override
	public String getNamespace() {
		return repository.getNamespace();
	}

	@Override
	public String getOntologyFileName() {
		return repository.getOntologyFileName();
	}

	@Override
	public Resource getResource() {
		return repository.getResource();
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((repository == null) ? 0 : repository.hashCode());
		result = prime * result + ((versionName == null) ? 0 : versionName.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		SAREFVersion other = (SAREFVersion) obj;
		if (repository == null) {
			if (other.repository != null)
				return false;
		} else if (!repository.equals(other.repository))
			return false;
		if (versionName == null) {
			if (other.versionName != null)
				return false;
		} else if (!versionName.equals(other.versionName))
			return false;
		return true;
	}

	
	
}
