/*
 * Copyright 2020 ETSI
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package fr.emse.gitlab.saref.managers;

import java.io.IOException;

import org.eclipse.jgit.api.Git;
import org.eclipse.jgit.api.errors.GitAPIException;
import org.eclipse.jgit.lib.Ref;
import org.slf4j.Logger;

import fr.emse.gitlab.saref.SAREFErrorLogger;
import fr.emse.gitlab.saref.SAREFPipeline;
import fr.emse.gitlab.saref.SAREFPipeline.Mode;
import fr.emse.gitlab.saref.SAREFPipelineException;
import fr.emse.gitlab.saref.checkers.Clause_9_2_Checker;
import fr.emse.gitlab.saref.checkers.Clause_9_3_Checker;
import fr.emse.gitlab.saref.checkers.Clause_9_4_Checker;
import fr.emse.gitlab.saref.checkers.Clause_9_5_Checker;
import fr.emse.gitlab.saref.checkers.Clause_9_6_Checker;
import fr.emse.gitlab.saref.entities.SAREFRepository;
import fr.emse.gitlab.saref.entities.SAREFVersion;
import fr.emse.gitlab.saref.entities.SAREFVersionName;

public class RepositoryManager extends SAREFErrorLogger {

	private static enum MESSAGE {
		reset, checkout;
	}

	private final SAREFRepository repository;
	private String currentBranch;
	private SAREFVersion currentVersion;
	private SAREFVersionName currentVersionName;

	public RepositoryManager(SAREFPipeline pipeline, Logger errorLogger, SAREFRepository repository) {
		super(pipeline, errorLogger);
		this.repository = repository;
		this.currentBranch = repository.getOriginalBranch();
		this.currentVersionName = repository.getOriginalVersion();
		this.currentVersion = repository.getVersion(currentVersionName);
	}
	
	public SAREFRepository getRepository() {
		return repository;
	}

	public String getCurrentBranch() {
		return currentBranch;
	}
	
	public SAREFVersion getCurrentVersion() {
		return currentVersion;
	}

	public SAREFVersionName getCurrentVersionName() {
		return currentVersionName;
	}

	public void check() throws SAREFPipelineException {
		new Clause_9_2_Checker(this).check();
		new Clause_9_3_Checker(this).check();
		new Clause_9_4_Checker(this).check();
		new Clause_9_5_Checker(this).check();
		new Clause_9_6_Checker(this).check();
	}
	
	public void checkoutVersion(SAREFVersionName versionName, boolean releaseOnly) throws SAREFPipelineException {
		Ref ref = getRef(versionName, releaseOnly);
		if(ref == null) {
			throw new SAREFPipelineException();
		}
		try (Git git = Git.open(repository.getDirectory())) {
			git.checkout().setName(ref.getName()).call();
			currentBranch = reg.getName();
			currentVersion = versionName;
		} catch (IOException | GitAPIException ex) {
			String msg = getMessage(MESSAGE.checkout, repository.getProject().getName(), versionName);
			log(msg, ex, Mode.DEVELOP, Mode.RELEASE, Mode.PORTAL);
			throw new SAREFPipelineException(msg, ex);
		}
	}
	
	private Ref getRef(SAREFVersionName versionName, boolean releaseOnly) {
		SAREFVersion versionRefs = repository.getVersions().get(versionName);
		if(versionRefs.getReleaseRef() != null) {
			return versionRefs.getReleaseRef();
		}
		if(releaseOnly) {
			return null;
		}
		if(versionRefs.getPrereleaseRef() != null) {
			return versionRefs.getPrereleaseRef();
		}
		return versionRefs.getDevelopRef();
	}


	public void resetCheckout() throws SAREFPipelineException {
		String originalBranch = repository.getOriginalBranch();
		if (originalBranch == null) {
			return;
		}
		try (Git git = Git.open(repository.getDirectory())) {
			git.checkout().setName(originalBranch).call();
		} catch (IOException | GitAPIException ex) {
			String msg = getMessage(MESSAGE.reset, originalBranch);
			log(msg, ex, Mode.DEVELOP, Mode.RELEASE, Mode.PORTAL);
			throw new SAREFPipelineException(msg, ex);
		}
	}

}
