/*
 * Copyright 2020 ETSI
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package fr.emse.gitlab.saref.managers;

import java.io.Console;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.io.FileUtils;
import org.apache.commons.text.StringSubstitutor;
import org.eclipse.jgit.api.Git;
import org.eclipse.jgit.transport.CredentialsProvider;
import org.eclipse.jgit.transport.UsernamePasswordCredentialsProvider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.dataformat.yaml.YAMLFactory;

import fr.emse.gitlab.saref.SAREF;
import fr.emse.gitlab.saref.SAREFErrorLogger;
import fr.emse.gitlab.saref.SAREFPipeline;
import fr.emse.gitlab.saref.SAREFPipeline.Mode;
import fr.emse.gitlab.saref.SAREFPipelineException;
import fr.emse.gitlab.saref.entities.SAREFCore;
import fr.emse.gitlab.saref.entities.SAREFExtension;
import fr.emse.gitlab.saref.entities.SAREFProject;
import fr.emse.gitlab.saref.entities.SAREFRepository;

/**
 *
 * @author Maxime Lefrançois
 *
 */
public class SourcesManager extends SAREFErrorLogger {

	private static final Logger LOG = LoggerFactory.getLogger(SourcesManager.class);
	private static final String CONFIGURATION_FILE_NAME = ".saref-repositories.yml";
	private static final String NAME_SOURCES = "sources";
	
	private static enum MESSAGE {
		versions, yaml_error, yaml_repository, yaml_fetch, yaml_failure, account_name,
	    account_name_empty, account_password, account_password_empty, reset_checkout_error, check_versions_error;
	}

	private final File sourcesDir;
	private final Map<SAREFProject, RepositoryManager> sourcesManagers = new HashMap<>();
	private final RepositoryManager targetRepositoryManager;

	public SourcesManager(SAREFPipeline pipeline, Logger errorLogger) throws SAREFPipelineException {
		super(pipeline, errorLogger);
		this.sourcesDir = new File(pipeline.targetDir, NAME_SOURCES);
		if(pipeline.mode == Mode.PORTAL) {
			targetRepositoryManager = null;
		} else {
			File directory = pipeline.directory;
			Logger repositoryLogger = pipeline.getLogger(getMessage(MESSAGE.versions, directory.getName()));
			RepositoryFactory repositoryFactory = new RepositoryFactory(pipeline, repositoryLogger, true);
			SAREFRepository repository = repositoryFactory.create(directory);
			targetRepositoryManager = new RepositoryManager(pipeline, repositoryLogger, repository);
		}
	}

	/**
	 * Fetch all the repositories in the configuration file.
	 * 
	 * @throws SAREFPipelineException
	 */
	public void fetchRepositories() throws SAREFPipelineException {	
		final File confFile = new File(pipeline.directory, CONFIGURATION_FILE_NAME);
		if (!confFile.exists()) {
			if (pipeline.mode == Mode.PORTAL) {
				log(getMessage(MESSAGE.yaml_error, CONFIGURATION_FILE_NAME), Mode.PORTAL);
			}
			return;
		}
		YAMLRepos repos;
		try {
			ObjectMapper mapper = new ObjectMapper(new YAMLFactory());
			repos = mapper.readValue(confFile, YAMLRepos.class);
		} catch (IOException ex) {
			log(getMessage(MESSAGE.yaml_error, CONFIGURATION_FILE_NAME), Mode.DEVELOP, Mode.RELEASE, Mode.PORTAL);
			return;
		}
		try {
			FileUtils.forceMkdir(sourcesDir);
		} catch (IOException ex) {
			log(getMessage(MESSAGE.yaml_repository, sourcesDir), ex, Mode.DEVELOP, Mode.RELEASE, Mode.PORTAL);
			return;
		}
		for (YAMLRepos.Entry<String, YAMLHost> entry : repos.entrySet()) {
			final String hostName = entry.getKey();
			final YAMLHost host = entry.getValue();
			fetchRepository(hostName, host);
		}
	}

	/**
	 * Fetches a repository and adds its manager to the map.
	 * 
	 * @param hostName
	 * @param host
	 * @throws SAREFPipelineException
	 */
	private void fetchRepository(String hostName, YAMLHost host) throws SAREFPipelineException {
		final CredentialsProvider credentialsProvider = getCredentialsProvider(hostName, host.credentials);
		for (String repo : host.repos) {
			final String name = repo.substring(repo.lastIndexOf("/") + 1);
			File repositoryDirectory = new File(sourcesDir, name);

			Logger repositoryLogger = pipeline.getLogger(getMessage(MESSAGE.versions, name));
			RepositoryFactory repositoryFactory = new RepositoryFactory(pipeline, repositoryLogger, true);
			SAREFRepository repository = repositoryFactory.create(repositoryDirectory);
			if(repository == null) {
				continue;
			}
			SAREFProject project = repository.getProject();
			if (repositoryDirectory.isDirectory()) {
				try (Git git = Git.open(repositoryDirectory)) {
					git.fetch().setCredentialsProvider(credentialsProvider).setRemoveDeletedRefs(true).call();
				} catch (Exception ex) {
					log(getMessage(MESSAGE.yaml_fetch, name), ex);
					continue;
				}
			} else {
				String http_url_to_repo = String.format("https://%s/%s.git", hostName, repo);
				try (Git git = Git.cloneRepository().setCredentialsProvider(credentialsProvider)
						.setURI(http_url_to_repo).setDirectory(repositoryDirectory).call()) {
				} catch (Exception ex) {
					log(getMessage(MESSAGE.yaml_failure, http_url_to_repo, name), ex);
					continue;
				}
			}
			RepositoryManager repositoryManager = new RepositoryManager(pipeline, repositoryLogger, repository);
			sourcesManagers.put(project, repositoryManager);
		}
	}

	private CredentialsProvider getCredentialsProvider(String hostName, YAMLCredentials credentials) {
		final String username;
		final char[] password;
		final Console console = System.console();
		if (credentials != null && credentials.username != null) {
			username = StringSubstitutor.replace(credentials.username, System.getenv());
		} else {
			if (console != null) {
				username = console.readLine(getMessage(MESSAGE.account_name, hostName));
			} else if (System.getenv("GITLAB_CI") != null) {
				LOG.info("using username 'gitlab-ci-token'");
				username = "gitlab-ci-token";
			} else {
				LOG.warn(getMessage(MESSAGE.account_name_empty));
				username = "";
			}
		}
		if (credentials != null && credentials.password != null) {
			password = StringSubstitutor.replace(credentials.password, System.getenv()).toCharArray();
		} else {
			if (console != null) {
				password = console.readPassword(getMessage(MESSAGE.account_password, hostName));
			} else if (System.getenv("GITLAB_CI") != null) {
				password = System.getenv("CI_JOB_TOKEN").toCharArray();
			} else {
				LOG.warn(getMessage(MESSAGE.account_password_empty));
				password = "".toCharArray();
			}
		}
		return new UsernamePasswordCredentialsProvider(username, password);
	}

	private static class YAMLRepos extends HashMap<String, YAMLHost> {
		private static final long serialVersionUID = 3434677850580166200L;
	}

	private static class YAMLHost {
		private YAMLCredentials credentials;
		private List<String> repos;

		@JsonCreator
		public YAMLHost(@JsonProperty(value = "credentials", required = false) YAMLCredentials credentials,
				@JsonProperty(value = "repos", required = false) List<String> repos) {
			this.credentials = credentials;
			this.repos = repos;
		}
	}

	private static class YAMLCredentials {
		private String username;
		private String password;

		@JsonCreator
		public YAMLCredentials(@JsonProperty(value = "username", required = false) String username,
				@JsonProperty(value = "password", required = false) String password) {
			this.username = username;
			this.password = password;
		}
	}

	public void check() throws SAREFPipelineException {
		if(targetRepositoryManager != null) {
			// Mode is DEVELOP or RELEASE.
			targetRepositoryManager.check();
		} else {
			// Mode is PORTAL. Check every source
			boolean hasErrors = false;
			for (RepositoryManager repositoryManager : sourcesManagers.values()) {
				try {
					repositoryManager.check();
				} catch(SAREFPipelineException ex) {
					hasErrors = true;
				}
			}
			if(hasErrors) {
				log(getMessage(MESSAGE.check_versions_error), Mode.DEVELOP, Mode.RELEASE, Mode.PORTAL);
			}
		}
	}

	public void resetCheckout() throws SAREFPipelineException {
		boolean hasErrors = false;
		try {
			targetRepositoryManager.resetCheckout();
		} catch(SAREFPipelineException ex) {
			hasErrors = true;
		}
		for (RepositoryManager repositoryManager : sourcesManagers.values()) {
			try {
				repositoryManager.resetCheckout();
			} catch(SAREFPipelineException ex) {
				hasErrors = true;
			}
		}		
		if(hasErrors) {
			log(getMessage(MESSAGE.reset_checkout_error), Mode.DEVELOP, Mode.RELEASE, Mode.PORTAL);
		}
	}
	
	public RepositoryManager findRepositoryManager(String uri) {
		final Matcher matcher = Pattern.compile(String.format("^%s%s/", SAREF.BASE, SAREFProject.REGEX_PATH_PROJECT)).matcher(uri);
		if (!matcher.find()) {
			return null;
		}
		final String acronym = matcher.group(SAREFProject.REGEX_ACRONYM_VAR);
		final SAREFProject project;
		if (acronym == null) {
			project = SAREFCore.INSTANCE;
		} else {
			project = new SAREFExtension(acronym);
		}
		return findRepositoryManager(project);
	}
	
	
	public RepositoryManager findRepositoryManager(SAREFProject project) {
		if(targetRepositoryManager.getRepository().getProject().equals(project)) {
			return targetRepositoryManager;
		}
		return sourcesManagers.get(project);
	}

}
