/*
 * Copyright 2020 ETSI
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package fr.emse.gitlab.saref.managers.parsers;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.nio.charset.Charset;
import java.nio.charset.MalformedInputException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.jena.atlas.RuntimeIOException;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.rdf.model.ModelFactory;
import org.apache.jena.riot.Lang;
import org.semanticweb.owlapi.formats.RDFDocumentFormat;
import org.semanticweb.owlapi.formats.TurtleDocumentFormatFactory;
import org.semanticweb.owlapi.io.OWLOntologyDocumentSource;
import org.semanticweb.owlapi.io.OWLParserException;
import org.semanticweb.owlapi.model.OWLDocumentFormat;
import org.semanticweb.owlapi.model.OWLDocumentFormatFactory;
import org.semanticweb.owlapi.model.OWLOntology;
import org.semanticweb.owlapi.model.OWLOntologyLoaderConfiguration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import fr.emse.gitlab.saref.SAREF;
import fr.emse.gitlab.saref.SAREFPipeline;
import fr.emse.gitlab.saref.SAREFPipeline.Mode;
import fr.emse.gitlab.saref.entities.SAREFCore;
import fr.emse.gitlab.saref.entities.SAREFExtension;
import fr.emse.gitlab.saref.entities.SAREFProject;
import fr.emse.gitlab.saref.entities.SAREFVersionName;
import fr.emse.gitlab.saref.managers.RepositoryManager;

/**
 * If the ontology IRI is that of a SAREF ontology version, attempt to checkout the appropriate branch of the corresponding source repository, and parse the ontology.
 * 
 * @author Maxime Lefrançois
 *
 */
public class PipelineModelOntologyParser extends AbstractJenaModelOntologyParser {
	static final Logger LOG = LoggerFactory.getLogger(PipelineModelOntologyParser.class);

	private static final String REGEX = String.format("^%s%s/%s/$", SAREF.BASE, SAREFProject.REGEX_PATH_PROJECT,
			SAREFVersionName.REGEX_VERSION_NUMBER);
	private static final Pattern PATTERN = Pattern.compile(REGEX);

	private final SAREFPipeline pipeline;

	public PipelineModelOntologyParser(SAREFPipeline pipeline) {
		this.pipeline = pipeline;
	}

	@Override
	public OWLDocumentFormatFactory getSupportedFormat() {
		return new TurtleDocumentFormatFactory();
	}

	@Override
	public OWLDocumentFormat parse(OWLOntologyDocumentSource source, OWLOntology ontology,
			OWLOntologyLoaderConfiguration config) {
		String iri = source.getDocumentIRI().toString();
		try {
			final Matcher matcher = PATTERN.matcher(iri);
			if (!matcher.find()) {
				throw new OWLParserException(
						"the iri " + iri + " does not correspond to a SAREF ontology version IRI.");
			}
			final String acronym = matcher.group(SAREFProject.REGEX_ACRONYM_VAR);
			final SAREFProject project;
			if (acronym == null) {
				project = SAREFCore.INSTANCE;
			} else {
				project = new SAREFExtension(acronym);
			}
			RepositoryManager repositoryManager = pipeline.getSourcesManager().findRepositoryManager(project);
			int major = Integer.parseInt(matcher.group(SAREFVersionName.REGEX_VERSION_MAJOR_VAR));
			int minor = Integer.parseInt(matcher.group(SAREFVersionName.REGEX_VERSION_MINOR_VAR));
			int patch = Integer.parseInt(matcher.group(SAREFVersionName.REGEX_VERSION_PATCH_VAR));
			final SAREFVersionName versionName = new SAREFVersionName(major, minor, patch);

			boolean releaseOnly = pipeline.mode == Mode.RELEASE||  pipeline.mode == Mode.PORTAL;
			repositoryManager.checkoutVersion(versionName, releaseOnly);
			File dir = new File(repositoryManager.getRepository().getDirectory(), "ontology");
			File file = new File(dir, repositoryManager.getRepository().getProject().getOntologyFileName());
			Model model = ModelFactory.createDefaultModel();
			try (FileInputStream input = new FileInputStream(file)) {
				model.read(input, SAREF.BASE, Lang.TTL.getLabel());
			} catch (Exception ex) {
				if (ex instanceof RuntimeIOException && ex.getCause() instanceof MalformedInputException) {
					try (InputStreamReader input2 = new InputStreamReader(new FileInputStream(file),
							Charset.defaultCharset())) {
						model.read(input2, SAREF.BASE, Lang.TTL.getLabel());
					} catch (Exception ex2) {
						throw ex2;

					}
				} else {
					throw ex;
				}
			}
			RDFDocumentFormat format = new PipelineModelFormat();
			parse(source, ontology, config, model, format);
			return format;
		} catch (Exception ex) {
			LOG.warn("Error while getting graph " + iri, ex);
			throw new OWLParserException("Error while getting graph " + iri, ex);
		}

	}

}
